from unittest import TestCase
from ftsa.cli.utils import sanitize, get_base_project_path, Caps


class FilesTest(TestCase):

    def setUp(self) -> None:
        self._myDirName = 'MyDír nâMè'

    def test_sanitize_standard(self):
        self.assertEqual(sanitize(self._myDirName), 'mydir-name')

    def test_sanitize_lowercase(self):
        self.assertEqual(sanitize(self._myDirName, Caps.LOWER), 'mydir-name')

    def test_sanitize_uppercase(self):
        self.assertEqual(sanitize(self._myDirName, Caps.UPPER), 'MYDIR-NAME')

    def test_sanitize_firstuppercase(self):
        self.assertEqual(sanitize(self._myDirName, Caps.FIRST), 'Mydir-Name')

    def test_sanitize_standard_change_blank_for_underline(self):
        self.assertEqual(sanitize(self._myDirName, inn=' ', out='_'), 'mydir_name')

    def test_sanitize_lowercase_change_blank_for_underline(self):
        self.assertEqual(sanitize(self._myDirName, Caps.LOWER, inn=' ', out='_'), 'mydir_name')

    def test_sanitize_uppercase_change_blank_for_underline(self):
        self.assertEqual(sanitize(self._myDirName, Caps.UPPER, inn=' ', out='_'), 'MYDIR_NAME')

    def test_sanitize_firstuppercase_change_blank_for_underline(self):
        self.assertEqual(sanitize(self._myDirName, Caps.FIRST, inn=' ', out='_'), 'Mydir_Name')

    def test_get_base_project_path(self):
        path = get_base_project_path()
        print(f'Path is: {path}')
        self.assertIsNotNone(path)


import ftsa.cli.modules.install_upgrade_parser as iup
import ftsa.cli.modules.run_report_parser as rep
import ftsa.cli.modules.init_parser as ini

from argparse import ArgumentParser


def main():
    parser = ArgumentParser()
    subparsers = parser.add_subparsers()

    ''' Install parser '''
    install_parser = subparsers.add_parser('install', help='Installs all framework dependencies.')
    install_parser.set_defaults(handler=iup.install)

    ''' Upgrade parser '''
    upgrade_parser = subparsers.add_parser('upgrade', help='Upgrades all framework dependencies.')
    upgrade_parser.set_defaults(handler=iup.upgrade)

    ''' Uninstall parser '''
    uninstall_parser = subparsers.add_parser('uninstall', help='Uninstalls all framework dependencies.')
    uninstall_parser.set_defaults(handler=iup.uninstall)

    ''' Report parser '''
    report_parser = subparsers.add_parser('report', help='Runs tests using Docker engine, recording (in case of '
                                                         '*web* ui) execution videos and generating execution '
                                                         'result report.')
    # report_parser.add_argument('-p', '--parallel', help='Number of parallel instances. Ex: 2 (two) etc.')
    report_parser.add_argument('-i', '--include', action='append',
                               help='Includes a tag to the execution. Ex: -i all')
    report_parser.add_argument('-e', '--exclude', action='append',
                               help='Excludes a tag of the execution. Ex: -e fe')
    report_parser.add_argument('-nd', '--nodocker', action='store_true', help='Docker execution ignored.')
    report_parser.set_defaults(handler=rep.report)

    ''' Docker Report parser'''
    docker_report_parser = subparsers.add_parser('docker-report', help='Runs tests using an external docker engine')
    docker_report_parser.add_argument('-i', '--include', action='append',
                                      help='Includes a tag to the execution. Ex: -i all')
    docker_report_parser.add_argument('-e', '--exclude', action='append',
                                      help='Excludes a tag of the execution. Ex: -e fe')
    docker_report_parser.add_argument('-n', '--net', action='append',
                                      help='Informs docker network (grid) name. '
                                           'Default: YYYYMMDD_grid (YYYY-year, MM-month, DD-day)')
    docker_report_parser.add_argument('-c', '--container', action='append',
                                      help='Informs the docker execution container name. '
                                           'Default: YYYYMMDD_execution_robot (YYYY-year, MM-month, DD-day)')
    docker_report_parser.add_argument('-p', '--pull', action='store_true',
                                      help='pulls the container image from Docker Hub although '
                                           'creating it locally using Dockerfile.')
    docker_report_parser.add_argument('-dh', '--dockerhost', action='append',
                                      help='Informs IP or docker host name address to docker client host machine. '
                                           'Default: localhost.')
    docker_report_parser.add_argument('-nd', '--nodocker', action='store_true', help='Docker execution ignored.')
    docker_report_parser.set_defaults(handler=rep.docker_report)

    ''' Report Clear parser '''
    report_clear_parser = subparsers.add_parser('clear', help='Limpar arquivos e diretórios de relatório.')
    report_clear_parser.set_defaults(handler=rep.clear)

    ''' Run parser'''
    run_parser = subparsers.add_parser('run', help='Executa o projeto FTSA.')
    run_parser.add_argument('-p', '--parallel', help='Número de instâncias paralelas. Ex: 2 (duas) etc.')
    run_parser.add_argument('-i', '--include', action='append',
                            help='Informa as tags que devem ser incluídas. Ex: all')
    run_parser.add_argument('-e', '--exclude', action='append',
                            help='Informa as tags que devem ser excluídas. Ex: fe')
    run_parser.set_defaults(handler=rep.run)

    ''' Init parser '''
    init_parser = subparsers.add_parser('init', help='Inicializa um projeto FTSA (TJPB, básico, com login).')
    init_parser.add_argument('project_name', help='Informa o NOME do projeto FTSA.')
    init_parser.add_argument('-e', '--example', action='store_true',
                             help='Cria um projeto para o FTSA-TJPB (com exemplo).')
    init_parser.add_argument('-s', '--services', action='store_true',
                             help='Cria um projeto para o FTSA-Services (testes REST / API ou XML).')
    init_parser.set_defaults(handler=ini.init)

    ''' Execute command with args '''
    args = parser.parse_args()
    args.handler(args)



import shutil
import os
from ftsa.cli.utils.files import sanitize, get_base_project_path, get_example_project_path, get_commons_project_path, \
                                 DIR_COMMONS, DIR_RESOURCES, FILE_PROJECT_PROPERTIES, is_project_ftsa, \
                                 get_service_project_path, FILE_GITIGNORE, FILE_MAIN_PY, FILE_DOCKERFILE


def init(args):
    if hasattr(args, 'project_name') and getattr(args, 'project_name') is not None:
        project_base_dir = sanitize(args.project_name)
        if hasattr(args, 'services') and getattr(args, 'services') is not None and getattr(args, 'services'):
            shutil.copytree(get_service_project_path(), project_base_dir)
        elif hasattr(args, 'example') and getattr(args, 'example') is not None and getattr(args, 'example'):
            shutil.copytree(get_example_project_path(), project_base_dir)
        else:
            shutil.copytree(get_base_project_path(), project_base_dir)
        os.chdir(project_base_dir)
        update(args, True)


def update(args, first=False):
    is_project_ftsa()
    if os.path.isdir(f'{DIR_COMMONS}'):
        shutil.rmtree(f'{DIR_COMMONS}')
    shutil.copytree(get_commons_project_path(), f'{DIR_COMMONS}')
    if first:
        shutil.move(f'{DIR_COMMONS}{os.sep}{FILE_PROJECT_PROPERTIES}',
                    f'{DIR_RESOURCES}{os.sep}{FILE_PROJECT_PROPERTIES}')
        shutil.move(f'{DIR_COMMONS}{os.sep}{FILE_GITIGNORE}',
                    f'{FILE_GITIGNORE}')
        shutil.move(f'{DIR_COMMONS}{os.sep}{FILE_MAIN_PY}',
                    f'{FILE_MAIN_PY}')
        shutil.move(f'{DIR_COMMONS}{os.sep}{FILE_DOCKERFILE}',
                    f'{FILE_DOCKERFILE}')
    if os.path.isdir(f'{DIR_COMMONS}'):
        shutil.rmtree(f'{DIR_COMMONS}')


PACKAGE_NAME  = 'ftsa-tjpb'
IMAGE_NAME    = 'testetjpb/ftsa-tjpb-image'
IMAGE_VERSION = '2.6.1'
VERSION       = '2.6.1'
TEST_PYPI     = False
DEPENDENCIES  = [
    'robotframework',
    'allure-python-commons',
    'allure-pytest',
    'allure-behave',
    'allure-robotframework',
    'robotframework-pabot',
    'robotframework-seleniumlibrary',
    'robotframework-appiumlibrary',
    'robotframework-faker',
    'robotframework-datadriver',
    'robotframework-databaselibrary',
    'robotframework-SikuliLibrary',
    'robotframework-requests',
    'robotframework-jsonlibrary',
    'robotframework-sshlibrary',
    'robotframework-pdf2textlibrary',
    'pyautogui',
    'selenium',
    'webdrivermanager',
    'opencv-python',
    'jproperties',
    'requests',
    'jsonpath_rw',
    'jsonpath_rw_ext',
    'pandas',
    'numpy',
    'PyMySQL',
    'psycopg2-binary',
    'cx_Oracle',
    'ftsa-tjpb-core'
]


import os
import platform
from ftsa.cli.modules.properties import DEPENDENCIES


BROWSERS = [
    'firefox',
    'chrome',
    'opera'
]


def run(up):
    for dependency in DEPENDENCIES:
        os.system(f'pip install {"--upgrade " if up else ""}{dependency}')
    for browser in BROWSERS:
        os.system(f'webdrivermanager {browser}')
    if platform.system() in 'Windows':
        os.system(f'webdrivermanager edge')
        os.system(f'webdrivermanager ie')
    os.system('npm install -g allure-commandline')
    os.system('npm install -g appium')


def install(args):
    run(False)


def upgrade(args):
    run(True)


def uninstall(args):
    for dependency in DEPENDENCIES:
        os.system(f'pip uninstall -y {dependency}')
    os.system(f'pip uninstall -y ftsa-tjpb')



import os
import socket
import docker
import platform
from datetime import datetime
from ftsa.cli.utils import execute
from ftsa.cli.utils.files import delete_path, delete_file, is_project_ftsa, get_project_properties
from ftsa.cli.modules.properties import IMAGE_NAME, IMAGE_VERSION


RESULTS_DIRECTORIES = [
    'results',
    'output',
    'pabot_results',
    'report',
    'dist',
    'videos'
]

FILES = [
    '.pabotsuitenames',
    'log.html',
    'report.html',
    'output.xml'
]


def add_tags(args, tipo):
    if hasattr(args, tipo) and getattr(args, tipo) is not None:
        return ' '.join(['--' + tipo + ' ' + i for i in getattr(args, tipo)])
    return ''


def docker_report(args):
    is_project_ftsa()
    clear(args)
    include = add_tags(args, 'include')
    exclude = add_tags(args, 'exclude')
    docker_client = docker.from_env()

    # Setting no-docker execution option
    if hasattr(args, 'nodocker') and getattr(args, 'nodocker'):
        os.environ['NO_DOCKER_EXECUTION'] = 'nodocker'
        report(args)
    else:
        os.environ['NO_DOCKER_EXECUTION'] = 'dockerallowed'
        if hasattr(args, 'pull') and getattr(args, 'pull'):
            os.system(f'docker pull {IMAGE_NAME}:{IMAGE_VERSION}')
        else:
            os.system(f'docker build -t {IMAGE_NAME}:{IMAGE_VERSION} ./')

        # Use NET param OR, if none, [timestamp]_grid will be set
        now = datetime.now().strftime("%Y%m%d%H%M%S")
        network_name = f'{now}_grid'
        if hasattr(args, 'net') and getattr(args, 'net') is not None:
            network_name = getattr(args, 'net')
        os.system(f'docker network create {network_name}')

        # Use CONTAINER param OR, if none, [network_name]_execution_robot
        user_exec_container = f'{network_name}_execution_robot'
        if hasattr(args, 'container') and getattr(args, 'container') is not None:
            user_exec_container = getattr(args, 'container')

        # Use DOCKERHOST param OR, if none, use current machine IP
        host = socket.gethostbyname(socket.gethostname())
        if hasattr(args, 'dockerhost') and getattr(args, 'dockerhost') is not None:
            host = getattr(args, 'dockerhost')

        # Creating path to save video files
        project_cwd = os.getcwd()
        videos_path = os.path.join(project_cwd, 'videos')
        if not os.path.isdir(videos_path):
            os.makedirs(videos_path)
        # Insert quotation marks if Windows dir
        print(f'Platform detected: {platform.system()}')
        if platform.system() == 'Windows':
            project_cwd = f'"{project_cwd}"'

        # Initializing volume to save video files
        video_recording_volume_name = 'video_recording_volume'
        videos_recorded_container = 'videos_recorded_container'
        try:
            videos_recorded = docker_client.containers.get(videos_recorded_container)
            videos_recorded.logs()
            videos_recorded.stop()
            videos_recorded.remove()
            print('Existing (old) videos recorded container removed.')
        except BaseException:
            print(f'Videos recorder container does not exist.')
        try:
            volume = docker_client.volumes.get(video_recording_volume_name)
            volume.remove()
            print('Existing (old) video recording volume removed.')
        except BaseException:
            print(f'Video recording volume does not exist.')
        cmd = f'docker volume create {video_recording_volume_name}'
        execute(cmd, 5)
        print(f'{video_recording_volume_name} volume created to save test executions.')

        # Initializing User Execution Independent container
        cmd = f'docker run --name {user_exec_container} --net {network_name} ' \
              f'-v /var/run/docker.sock:/var/run/docker.sock:rw ' \
              f'-v {project_cwd}:/opt/robotframework/tests:rw ' \
              f'-e SELENIUM_EXEC_HOST={host} ' \
              f'-e SELENIUM_CONTAINER_NAME={now}_selenium ' \
              f'-e VIDEOS_RECORDER_CONTAINER_NAME={now}_videos ' \
              f'-e USER_NETWORK_NAME={network_name} ' \
              f'-e USER_EXEC_CONTAINER_NAME={user_exec_container} ' \
              f'-e VOLUME_VIDEO_RECORDING_NAME={video_recording_volume_name} ' \
              f'{IMAGE_NAME}:{IMAGE_VERSION} ' \
              f'/bin/bash -c "ftsa report {include}{exclude}"'
        execute(cmd, 5)

        # os.system(f'docker stop {user_exec_container} && docker rm {user_exec_container}')
        # Finalize securely independent execution container
        try:
            exec_container = docker_client.containers.get(user_exec_container)
            exec_container.logs()
            exec_container.stop()
            exec_container.remove()
            print(f'{user_exec_container} container stopped and removed.')
        except BaseException:
            print(f'There is no {user_exec_container} container available OR already stopped and removed.')

        # os.system(f'docker network rm {network_name}')
        # Finalize securely grid network
        try:
            network = docker_client.networks.get(network_name)
            network.remove()
            print(f'{network} network removed.')
        except BaseException:
            print(f'There is no {network_name} network available OR already removed.')

        # Copy video files recorded to project videos path and exclude volume
        # os.system(f'docker volume rm video_recording_volume')
        try:
            cmd = f'docker run -it -d --name {videos_recorded_container} ' \
                  f'-v {video_recording_volume_name}:/videos fedora '
            execute(cmd, 5)
            videos_recorded = docker_client.containers.get(videos_recorded_container)
            videos_recorded.logs()
            cmd = f'docker cp {videos_recorded.id}:/videos {project_cwd}'
            print(f'Saving videos recorded to "{videos_path}" project path directory.')
            execute(cmd, 5)
            videos_recorded.stop()
            videos_recorded.remove()
            print(f'{videos_recorded_container} container removed.')
            volume = docker_client.volumes.get(video_recording_volume_name)
            volume.remove()
            execute('docker volume prune -f', 5)
            print(f'{video_recording_volume_name} volume removed.')
        except BaseException:
            print(f'There is no videos recorded available OR already removed.')


def report(args):
    is_project_ftsa()
    clear(args)
    include = add_tags(args, 'include')
    exclude = add_tags(args, 'exclude')

    # Setting no-docker execution option
    if hasattr(args, 'nodocker') and getattr(args, 'nodocker'):
        os.environ['NO_DOCKER_EXECUTION'] = 'nodocker'
    else:
        os.environ['NO_DOCKER_EXECUTION'] = 'dockerallowed'

    parallel = 1
    if hasattr(args, 'parallel') and getattr(args, 'parallel') is not None and int(getattr(args, 'parallel')) > 1:
        parallel = int(args.parallel)
    project = get_project_properties()
    try:
        quantity = project.get('prop', 'PARALLEL')
        if quantity is not None and int(quantity) > 1:
            parallel = int(project.get('prop', 'PARALLEL'))
            print(f'A propriedade PARALLEL foi definida no arquivo de propriedades: {parallel}')
    finally:
        print('A propriedade PARALLEL não existe no arquivo de propriedades do projeto')

    options = f'--verbose --testlevelsplit'
    if hasattr(args, 'allure') and getattr(args, 'allure') is not None and getattr(args, 'allure'):
        os.system(f'pabot {options} --processes {parallel} --listener allure_robotframework '
                  f'--outputdir ./output {include}{exclude} --timestampoutputs ./features')
        os.system(f'allure serve ./output/allure')
    elif parallel == 1:
        os.system(f'robot -L TRACE -d ./output {include}{exclude} --timestampoutputs ./features')
    else:
        os.system(f'pabot {options} --processes {parallel} --outputdir ./output {include}{exclude} --timestampoutputs ./features')


def clear(args):
    is_project_ftsa()
    for directory in RESULTS_DIRECTORIES:
        try:
            delete_path(os.path.join(os.getcwd(), directory))
        except BaseException:
            print(f'Path/Directory "{directory}" can NOT be deleted...')
    for file in FILES:
        try:
            delete_file(os.path.join(os.getcwd(), file))
        except BaseException:
            print(f'File "{file}" can NOT be deleted...')


def run(args):
    report(args)
    clear(args)



class MyLibCalculadora(object):

    def __init__(self):
        self._result = None

    def some(self, a, b):
        self._result = float(a) + float(b)
        print(f'Operando a soma dos valores de {a} e ${b}')

    def multiplique(self, a, b):
        self._result = float(a) * float(b)
        print(f'Operando a multiplicação dos valores de {a} por ${b}')

    def o_resultado_deve_ser(self, esperado):
        if self._result != float(esperado):
            raise AssertionError(f'O resultado da operação {self._result} é diferente do esperado: {esperado}')
        print(f'O resultado da operação {self._result} é o esperado: {esperado}')

    def o_resultado_nao_deve_ser(self, valor):
        if self._result == float(valor):
            raise AssertionError(f'O resultado da operação {self._result} deve ser diferente do valor: {valor}')
        print(f'O resultado da operação {self._result} é diferente do valor: {valor}')



from setuptools import setup, find_packages
from ftsa.cli.modules.properties import VERSION, PACKAGE_NAME, DEPENDENCIES

with open('README.md', 'r', encoding='utf-8') as fh:
    long_description = fh.read()

setup(
    name=PACKAGE_NAME,
    version=VERSION,
    author='Carlos Diego Quirino Lima',
    author_email='diegoquirino@gmail.com',
    description='Framework de Teste de Sistemas Automatizado',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='http://gitlab.tjpb.jus.br/testes/ftsa-tjpb.git',
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',
    keywords='ftsa framework teste sistemas automatizado tjpb',
    packages=find_packages(exclude=['tests*']),
    package_data={'ftsa': [
        'config/*.properties',
        'cli/projects/**/*',
        'cli/projects/example_project/**/*',
        'cli/projects/example_project/data/**/*',
        'cli/projects/example_project/features/**/*',
        'cli/projects/base_project/**/*',
        'cli/projects/base_project/data/**/*',
        'cli/projects/base_project/features/**/*',
        'cli/projects/services_project/**/*',
        'cli/projects/services_project/data/**/*',
        'cli/projects/services_project/features/**/*',
        'cli/projects/commons/**/*',
        'cli/projects/commons/.gitignore',
        'cli/projects/services_project/**/*'
    ]},
    install_requires=[
        'docker',
        'setuptools',
        'wheel',
        'twine',
    ],
    entry_points={'console_scripts': [
        'ftsa = ftsa.cli.main:main'
    ]},
    platforms='windows linux',
)



import os
import sys
import shutil
import unicodedata
import site
from enum import Enum
from configparser import ConfigParser
from ftsa.cli.exceptions import NotFTSAProjectException

""" Diretórios da aplicação """
DIR_DATA = 'data'
DIR_IMAGES = f'{DIR_DATA}{os.sep}img'
DIR_FEATURES = 'features'
DIR_PAGE_OBJECTS = 'page_objects'
DIR_RESOURCES = 'resources'
DIR_COMMONS = 'commons'
FILE_PROJECT_PROPERTIES = 'project.properties'
FILE_GITIGNORE = '.gitignore'
FILE_MAIN_PY = 'main.py'
FILE_DOCKER_COMPOSE = 'docker-compose.yml'
FILE_DOCKERFILE = 'Dockerfile'


def get_os_lib_base_dir():
    for path in site.getsitepackages():
        if 'site-packages' in path or 'dist-packages' in path:
            if os.path.isdir(os.path.join(path, 'ftsa')):
                return os.path.join(path, f'ftsa{os.sep}cli{os.sep}projects{os.sep}')
    for path, name, file in os.walk('/'):
        if 'site-packages' in path or 'dist-packages' in path:
            if os.path.isdir(os.path.join(path, 'ftsa')):
                return os.path.join(path, f'ftsa{os.sep}cli{os.sep}projects{os.sep}')
    return None


OS_LIB_BASE_DIR = get_os_lib_base_dir()
DIR_LIB_BASE_PROJECT = f'{OS_LIB_BASE_DIR}base_project'
DIR_LIB_SERVICE_PROJECT = f'{OS_LIB_BASE_DIR}services_project'
DIR_LIB_EXAMPLE_PROJECT = f'{OS_LIB_BASE_DIR}example_project'
DIR_LIB_COMMONS = f'{OS_LIB_BASE_DIR}{DIR_COMMONS}'
DIR_LIB_PROP_FILE = f'{OS_LIB_BASE_DIR}{DIR_COMMONS}{os.sep}{FILE_PROJECT_PROPERTIES}'


class Caps(Enum):
    FIRST = 1
    UPPER = 2
    LOWER = 3


def delete_path(path):
    if os.path.isdir(path):
        shutil.rmtree(path)


def delete_file(file):
    if os.path.isfile(file):
        os.remove(file)


def is_project_ftsa():
    if not os.path.isdir(os.path.join(os.getcwd(), DIR_DATA)) and \
            not os.path.isdir(os.path.join(os.getcwd(), DIR_IMAGES)) and \
            not os.path.isdir(os.path.join(os.getcwd(), DIR_FEATURES)) and \
            not os.path.isdir(os.path.join(os.getcwd(), DIR_PAGE_OBJECTS)) and \
            not os.path.isdir(os.path.join(os.getcwd(), DIR_RESOURCES)):
        raise NotFTSAProjectException('Você deve estar em um projeto FTSA para executar esta operação.')
    else:
        return True


def get_base_project_path():
    return DIR_LIB_BASE_PROJECT


def get_service_project_path():
    return DIR_LIB_SERVICE_PROJECT


def get_example_project_path():
    return DIR_LIB_EXAMPLE_PROJECT


def get_commons_project_path():
    return DIR_LIB_COMMONS


def get_commons_project_properties_file():
    return DIR_LIB_PROP_FILE


def get_project_properties():
    project = ConfigParser()
    project.read(os.path.join(os.getcwd(), DIR_RESOURCES, FILE_PROJECT_PROPERTIES))
    return project


def get_properties_file(arquivo):
    prop = ConfigParser()
    prop.read(os.path.join(os.getcwd(), DIR_RESOURCES, f'{arquivo}.properties'))
    return prop


def sanitize(name, caps=Caps.LOWER, inn=' ', out='-'):
    name = unicodedata.normalize('NFD', name)
    name = name.encode('ascii', 'ignore')
    name = name.decode('utf-8')
    if caps == Caps.FIRST:
        name = name.title()
    elif caps == Caps.UPPER:
        name = name.upper()
    else:
        name = name.lower()
    return ''.join([c if c not in inn else out for c in name])
